/*
 * Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * This file is part of the FSG PAPI Project
 *
 * The FSG PAPI Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FSG PAPI Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with the FSG PAPI Project; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: lpr.c,v 1.1.1.1 2002/09/18 19:30:13 jacobs Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <libintl.h>
#include <papi.h>

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout, "Usage: %s [-P printer] [-# copies] [-C class] [-J job] [-T title] [-p [-i indent] [-w width]] [-1|-2|-3|-4 font] [-m] [-h] [-s] [-filter_option] [file ..]\n", name);
	exit (1);
}

int
main(int ac, char *av[])
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_attribute_t **list = NULL;
	papi_job_t job = NULL;
	int exit_code = 0;
	char *printer = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	int remove = 0;
	int sym_link = 0;
	int c;

	while ((c = getopt(ac, av,
			"P:#:C:J:T:w:i:hplrstdgvcfmn1:2:3:4:")) != EOF)
		switch (c) {
		case 'P':
			printer = optarg;
			break;
		case '#':
			papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL,
					"copies", atoi(optarg));
			break;
		case 'C':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"rfc-1179-class", optarg);
			break;
		case 'J':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"job-name", optarg);
			break;
		case 'T':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"pr-title", optarg);
			break;
		case 'p':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/x-pr");
			papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL,
					"pr-filter", 1);
			break;
		case 'i':
			papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL,
					"pr-indent", atoi(optarg));
			break;
		case 'w':
			papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL,
					"pr-width", atoi(optarg));
			break;
		case 'h':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"job-sheets", "none");
			break;
		case 'l':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/octet-stream");
			break;
		case 'o':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/postscript");
			break;
		case 'c':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/x-cif");
			break;
		case 'd':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/x-dvi");
			break;
		case 'f':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/x-fortran");
			break;
		case 'g':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/x-plot");
			break;
		case 'n':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/x-ditroff");
			break;
		case 't':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/x-troff");
			break;
		case 'v':
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/x-raster");
			break;
		case 'm':
			papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL,
				"rfc-1179-mail", 1);
			break;
		case 'r':
			remove = 1;
			break;
		case 's':
			sym_link = 1;
			break;
		default:
			usage(av[0]);
		}

	if ((printer == NULL) &&
	    ((printer = getenv("PRINTER")) == NULL) &&
	    ((printer = getenv("LPDEST")) == NULL))
		printer = "lp";

	papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL, "copies", 1);
	papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/octet-stream");
	papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"job-sheets", "standard");
			

	status = papiServiceCreate(&svc, printer, NULL, NULL, NULL,
					encryption, NULL);
	if (status != PAPI_OK) {
		char *mesg = papiServiceGetStatusMessage(svc);

		if (mesg == NULL)
			mesg = papiStatusString(status);

		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), printer,
			mesg);
		exit(1);
	}

	status = papiJobSubmit(svc, printer, (const papi_attribute_t **)list,
				NULL, (const char **)&av[optind], &job);

	if (status != PAPI_OK) {
		char *mesg = papiServiceGetStatusMessage(svc);
		if (mesg == NULL) {
			mesg = papiStatusString(status);
			exit_code = 1;
		} else
			mesg = "failed";
		fprintf(stderr, "%s: %s\n", printer, mesg);
	}
	
	papiServiceDestroy(svc);

	exit(exit_code);
}
